// User.cs
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/> or 
// write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, 
// Boston, MA 02111-1307 USA
//

using System;
using System.Collections;
using System.Collections.Generic;

using Do.Universe;

namespace Skype {

  ////////////////////////////////////////////////////
  // Item Source and Action

  public class UserItemSource : IItemSource {

    List<IItem> items;

    public UserItemSource () {
      items = new List<IItem> ();
    }
                
    public string Name {
      get { return "Skype Users"; }
    }
		
    public string Description {
      get { return "Skype Users"; }
    }
		
    public string Icon {
      get { return "skype"; }
    }
                
    public Type[] SupportedItemTypes {
      get {
        return new Type[] { typeof (ContactItem), };
      }
    }

    public ICollection<IItem> Items {
      get { return items; }
    }
		
    public ICollection<IItem> ChildrenOfItem (IItem item) {
      return null;
    }
		
    public void UpdateItems () {
      items.Clear();
      try {
        User[] us = SkypeAPI.Instance.GetAllContactUsers();
        if (us == null) return;
        foreach (User i in us) {
			ContactItem c = null;
			if (!string.IsNullOrEmpty (i.Fullname))
				c = ContactItem.CreateWithName (i.Fullname);
			else if (!string.IsNullOrEmpty (i.DisplayName))
				c = ContactItem.CreateWithName (i.DisplayName);
			else
				c = ContactItem.CreateWithName (i.Handle);
			c ["skype.handle"] = i.Handle;
        	items.Add (c);
        }
      } catch (Exception e) {
        Console.WriteLine("Exception: User.cs :"+e);
      }
      Console.WriteLine("Users : "+items.Count);
    }

  }

  public class StartChatAction : AbstractAction {
    
    public StartChatAction() {
    }

    public override string Name {
      get { return "Start Skype Chat"; }
    }

    public override string Description {
      get { return "Start a new chat with a Skype contact."; }
    }

    public override string Icon {
      get { return SkypeAPI.RES("Message_128x128.png"); }
    }
    
    public override bool SupportsItem (IItem item) {
     	return null != (item as ContactItem) ["skype.handle"];
    }
    
    public override Type[] SupportedItemTypes {
      get {
        return new Type[] { typeof (ContactItem) };
      }
    }

    public override bool SupportsModifierItemForItems (IItem[] items, IItem modItem) {
      return false;
    }

    public override IItem[] Perform (IItem[] items, IItem[] modItems) {
      ContactItem i = items [0] as ContactItem;
      SkypeAPI.Instance.StartChat (i ["skype.handle"]);
      return null;
    }

  }

  public class StartCallAction : AbstractAction {
    
    public StartCallAction() {
    }

    public override string Name {
      get { return "Start Skype Call"; }
    }

    public override string Description {
      get { return "Start a new Skype call with a contact."; }
    }

    public override string Icon {
      get { return SkypeAPI.RES("CallStart_128x128.png"); }
    }
    
    public override bool SupportsItem (IItem item) {
     	return null != (item as ContactItem) ["skype.handle"];
    }
    
    public override Type[] SupportedItemTypes {
      get {
        return new Type[] { typeof (ContactItem) };
      }
    }

    public override bool SupportsModifierItemForItems (IItem[] items, IItem modItem) {
      return false;
    }

    public override IItem[] Perform (IItem[] items, IItem[] modItems) {
      ContactItem i = items [0] as ContactItem;
      SkypeAPI.Instance.StartCall (i ["skype.handle"]);
      return null;
    }

  }

}
