//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.

using System;
using System.Collections.Generic;

using Do.Universe;

namespace SqueezeCenter
{

	public class EnqueueAction : IAction
	{

		public EnqueueAction ()
		{
		}

		public string Name {
			get { return "Add to Play Queue"; }
		}

		public string Description {
			get { return "Add an item to the Squeezebox play queue."; }
		}

		public string Icon {
			get { return "add"; }
		}

		public Type[] SupportedItemTypes {
			get {
				return new Type[] {
					typeof (MusicItem),
				};
			}
		}
		
		public bool ModifierItemsOptional
		{
			get {
				return Server.Instance.GetConnectedPlayers ().Count == 0; 
			}
		}
		
		public IItem[] DynamicModifierItemsForItem(IItem item) 
		{
			// this converts converts the list of players to an array of IItem
			return Server.Instance.GetConnectedPlayersAsIItem ().ToArray ();
		}
		
		public bool SupportsModifierItemForItems (IItem[] items, IItem modifier)
		{
			return true;
		}
		
		public Type[] SupportedModifierItemTypes 
		{
			get {
				return new Type[] {
					typeof (Player),
				};		
			}
		}
		
		public bool SupportsItem (IItem item)
		{
			return (item is MusicItem);
		}

		public IItem[] Perform (IItem[] items, IItem[] modifierItems)
		{
			Player player;			
						
			if (modifierItems.Length > 0) {
				player = (modifierItems[0] as Player);
			}
			else {
				List<Player> availablePlayers = Server.Instance.GetConnectedPlayers ();
				if (availablePlayers.Count > 0) {
					player = availablePlayers[0];				
				}
				else {
					throw new Exception("Could not enqueue items. No player found");
				}
			}

			Server.Instance.AddItemsToPlayer (player, Util.Cast<IItem, MusicItem>(items));			
			return null;
		}
	}
}

