/* TSClientItemSource.cs
 *
 * GNOME Do is the legal property of its developers. Please refer to the
 * COPYRIGHT file distributed with this source distribution.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

using System;
using System.IO;
using System.Xml;
using System.Collections.Generic;

using Do;
using Do.Universe;

namespace Simulacra
{
	public class TSClientItemSource : IItemSource
	{
		List<IItem> items;

		public TSClientItemSource () {
			items = new List<IItem> ();
		}

		public string Name { get { return "Terminal Server Connection Items"; } }
		public string Description { get { return "Parses Connections in ~/.tsclient"; } }
		public string Icon { get { return "network-server"; } }

		public Type[] SupportedItemTypes {
			get {
				return new Type[] { typeof (TSClientItem) };
			}
		}

		public ICollection<IItem> Items {
			get { return items; }
		}

		public ICollection<IItem> ChildrenOfItem (IItem parent) {
			return null;
		}

		public void UpdateItems () {
			items.Clear ();
			try {
				string rdpFiles = Paths.Combine (Paths.UserHome, ".tsclient");
				DirectoryInfo di = new DirectoryInfo (rdpFiles);
				foreach (FileInfo fi in di.GetFiles ("*.rdp")) {
					string name = fi.Name.Replace (".rdp", "");
					items.Add (new TSClientItem (name, fi.FullName));
				}
			} catch { }
		}
	}

}
